<?php

namespace app\admin\controller\student;

use app\admin\model\department\Department;
use app\admin\model\Fault;
use app\admin\model\Report;
use app\common\controller\Backend;
use app\common\model\Config as ConfigModel;
use think\Db;
use think\Env;
use think\exception\PDOException;
use think\exception\ValidateException;

/**
 * sim-自主练习集合管理
 *
 * @icon fa fa-circle-o
 */
class Exercise extends Backend
{

    /**
     * Collection模型对象
     * @var \app\admin\model\teacher\Collection
     */
    protected $model = null;
    protected $exam_model = null;

    protected $whereExtend = null;
    public function _initialize()
    {
        parent::_initialize();
        $this->model = new \app\admin\model\teacher\Collection;
        $this->exam_model = new \app\admin\model\teacher\Exams;
        $this->assign('sim_sim_type', ConfigModel::getSimTypeList());
    }



    /**
     * 默认生成的控制器所继承的父类中有index/add/edit/del/multi五个基础方法、destroy/restore/recyclebin三个回收站方法
     * 因此在当前控制器中可不用编写增删改查的代码,除非需要自己控制这部分逻辑
     * 需要将application/admin/library/traits/Backend.php中对应的方法复制到当前控制器,然后进行修改
     */

    public function index()
    {
        //设置过滤方法
        $this->request->filter(['strip_tags', 'trim']);
        if (false === $this->request->isAjax()) {
            return $this->view->fetch();
        }
        if ($this->request->isPost()) {
            $params = $this->request->post('row/a');
            $count = 0;
            Db::startTrans();
            try {
                $params['exam_collection_type'] = 2;
                $params['exam_collection_state'] = 2;
                $params['question_setting_method'] = 1;
                $params['exam_collection_name'] = $this->auth->nickname.'自主练习';
                $params['limit_duration'] = 30;
                $params['start_time'] = date('Y-m-d');
                $params['end_time'] = date('Y-m-d');
                $exam_id = $this->model->insertGetId($params);

                $sim = Db::name('sim')->where('sim_type',$params['sim_type'])->where('seat_id',$this->auth->seat_id)->find();
                if(empty($sim)){
                    $this->error('未找到模拟器,不可练习');
                }

                $data = [
                    'exam_collection_id' => $exam_id,
                    'user_id' => $this->auth->id,
                    'user_username' => $this->auth->username,
                    'user_nickname' => $this->auth->nickname,
                    'user_depart_id' => $this->auth->depart_id,
                    'exam_collection_name' => $params['exam_collection_name'],
                    'exam_collection_type' =>$params['exam_collection_type'],
                    'sim_type' => $params['sim_type'],
                    'seat_id' =>$this->auth->seat_id,
                    'sim_id' =>$sim['sim_id'],
                    'exam_status'=>4,
                    'start_time'=>date('Y-m-d H:i:s'),
                    'starttime'=>time(),
                    'create_time'=>date('Y-m-d H:i:s'),
                    'createtime'=>time(),
                    'update_time'=>date('Y-m-d H:i:s'),
                    'updatetime'=>time(),
                ];
                $examid = $this->exam_model->insertGetId($data);


                //匹配的学员自动加入  mx_real_exam_fault 考试故障关联表
                $fault_list = Fault::where(['sim_type' => $params['sim_type'], 'fault_state' => 0,'fault_type'=>3])->select();
                foreach ($fault_list as $key=> $item1){
                    // $fault_key = [1,5,7];//array_rand($fault_list, 3);
                    $fault_key1 = rand(0, 1);
                    $fault_key2 = rand(2, 3);
                    $fault_key3 = rand(4, 5);
                    $fault_key = [$fault_key1,$fault_key2,$fault_key3];
                    $flag = 0;
                    if(!empty($fault_key) && in_array($key,$fault_key)){
                        $flag = 1;
                    }
                    $add= [
                        'exam_id'=>$examid,
                        'fault_id'=>$item1['fault_id'],
                        'ref_type'=>2,
                        'flag'=>$flag,
                        'ref_state'=>0,
                        'createtime'=>time(),
                        'updatetime'=>time(),
                        'create_time'=>date('Y-m-d H:i:s'),
                        'update_time'=>date('Y-m-d H:i:s'),
                    ];
                    Db::name('real_exam_fault')->insert($add);
                }

                $count = $examid;


                Db::commit();
            } catch (PDOException|Exception $e) {
                Db::rollback();
                $this->error($e->getMessage());
            }
            if ($count) {
                $this->success('开始成功','/admin/student/exercise/examing/ids/'.$count);
            }
            $this->error(__('No rows were updated'));
        }
    }

    public  function  examing($ids = null)
    {
        $row = $this->exam_model->get($ids);
        if(!$row){
            $this->error('未找到记录');
        }
        if ($this->request->isPost()) {

            //先请求接口判断，再进行处理
            if(Env::get('app.is_fault')){
                $url = config('site.url_type').'/sim/real-exam/student/exam/submit/'.$ids;
                // /dev-api/sim/real-exam/student/exam/submit/{examId}
                $ret = json_decode(send_get($url),true);
                // halt($ret);
                if($ret['code']!=200){
                    $this->error($ret['msg']);
                }
            }

            //计算分数，保存记录
            $params = $this->request->post('row/a');

            $result = false;
            $score = 100;
            $fault_one_score = 25;
            $fault_two_score = 25;
            $fault_three_score = 25;

            $xianxian_score = 0;
            $yuanyin_socre= 0;
            $buwei_score= 0;
            $fangfa_score= 0;

            $xianxian_arr = [];
            $yuanyin_arr = [];
            $buwei_arr = [];
            $fangfa_arr = [];
            $jielun_score = 0;


            Db::startTrans();
            try {

                //更新考试结束时间
                $row->end_time = date('Y-m-d H:i:s');
                $row->endtime = time();
                $row->exam_status = 5;
                $row->other_report = $params['other_report'];
                $result = $row->save();

                $info = $this->exam_model->where(['exam_id'=>$ids])->find();


                //更新故障是否正确
                $fault_list = Db::name('real_exam_fault')->where(['exam_id'=>$ids,'flag'=>1])->select();
                if(!empty($fault_list)){
                    foreach ($fault_list as $k =>$t){
                        $answer_right = 2;
                        if(!empty($t['sim_fault_question_value']) && !empty($t['sim_fault_answer_value'])){
                            if($t['sim_fault_question_value']!=$t['sim_fault_answer_value'] ){
                                //真实故障id 0002GZBW0001 薄膜开关FPC排线
                                if($t['sim_fault_answer_value']=='00000000' && $t['fault_id'] !='0002GZBW0001'){
                                    $answer_right=2;
                                }
                                $answer_right=1;
                            }
                        }
                        Db::name('real_exam_fault')->where(['ref_id'=>$t['ref_id']])->update(['answer_right'=>$answer_right]);
                    }
                }


                //计算得分，故障是否有扣分  扣分制
                $fault_right_list = Db::name('real_exam_fault')->where(['exam_id'=>$ids,'flag'=>1,'answer_right'=>['>',0]])->select();
                if(!empty($fault_right_list)){
                    if($fault_right_list[0]['answer_right']==1){
                        $fault_one_score = 0;
                    }
                    if($fault_right_list[1]['answer_right']==1){
                        $fault_two_score = 0;
                    }
                    if($fault_right_list[2]['answer_right']==1){
                        $fault_three_score = 0;
                    }
                }
                    
                $fault_right_list = Db::name('real_exam_fault')->where(['exam_id'=>$ids,'flag'=>1])->select();

                //根据故障部位,查找故障现象
                $fault_arr = [$fault_right_list[0]['fault_id'],$fault_right_list[1]['fault_id'],$fault_right_list[2]['fault_id']];
                
                //根据故障部位查找，匹配的排除方法
                $paichu_arr = Report::where(['bw_id'=>['in',$fault_arr],'sim_type'=>$info['sim_type']])->column('pc_id');
                // //根据故障部位查找，匹配可能原因
                // $yy_arr = Report::where(['bw_id'=>['in',$fault_arr],'sim_type'=>$info['sim_type']])->column('yy_id');

                //查找故障现象 数组
                $partent_fault_id = Fault::where(['fault_id'=>['in',$fault_arr],'sim_type'=>$info['sim_type']])->column('parent_fault_id');

                //根据故障部位查找，匹配可能原因
                $yy_arr = Fault::where(['parent_fault_id'=>['in',$partent_fault_id],'sim_type'=>$info['sim_type'],'fault_type'=>2])->column('fault_id');

                //获取数组
                $other_report = json_decode($params['other_report'],true);
                if(!empty($other_report)){
                    foreach ($other_report as $key => $em){
                        //故障现象
                        $xianxian_arr[] = $em['xx_id'];
                        //故障部位
                        $buwei_arr[] = $em['bw_id'];
                        //可能原因
                        foreach(explode(',',$em['yy_id']) as $k3 =>$t13){
                            $yuanyin_arr[] = $t13;
                        }
                        //排除方法
                        $fangfa_arr[] = $em['pc_id'];
                    }
                }
                
                //故障现象 分数 start
                foreach ($partent_fault_id as $k1 =>$t){
                    if(!in_array($t,$xianxian_arr)){
                        $xianxian_score = 1+$xianxian_score;
                        $xianxian_content[] = [
                            'cx_id'=>$t,
                            'cx_type'=>'少写',
                            'cx_name'=>Fault::where(['fault_id'=>$t])->value('name'),
                        ];
                    }
                }
                foreach ($xianxian_arr as $k2 =>$t2){
                    if(!in_array($t2,$partent_fault_id)){
                        $xianxian_score = 1+$xianxian_score;
                        $xianxian_content[] = [
                            'cx_id'=>$t2,
                            'cx_type'=>'错写',
                            'cx_name'=>Fault::where(['fault_id'=>$t2])->value('name'),
                        ];
                    }
                }
                //故障现象 end
              
                
                //故障部位 start
                foreach ($fault_arr as $k11 =>$t11){
                    if(!in_array($t11,$buwei_arr)){
                        $buwei_score = 1+$buwei_score;
                        $buwei_content[] = [
                            'cx_id'=>$t11,
                            'cx_type'=>'少写',
                            'cx_name'=>Fault::where(['fault_id'=>$t11])->value('name'),
                        ];
                    }
                }
                foreach ($buwei_arr as $k12 =>$t12){
                    if(!empty($t12) && !in_array($t12,$fault_arr)){
                        //错写
                        $buwei_score = 1+$buwei_score;
                        $buwei_content[] = [
                            'cx_id'=>$t12,
                            'cx_type'=>'错写',
                            'cx_name'=>Fault::where(['fault_id'=>$t12])->value('name'),
                        ];
                    }
                }
                //故障部位 end
                
                //排除方法 start
                //循环固定的排除方法，匹配作答的排除方法
                foreach ($paichu_arr as $k21 =>$t21){
                    if(!in_array($t21,$fangfa_arr)){
                        $fangfa_score = 1+$fangfa_score;
                        $fangfa_content[] = [
                            'cx_id'=>$t21,
                            'cx_type'=>'少写',
                            'cx_name'=>Fault::where(['fault_id'=>$t21])->value('name'),
                        ];
                    }
                }
                //循环作答的排除方法，，匹配固定的排除方法
                foreach ($fangfa_arr as $k22 =>$t22){
                    if(!empty($t22) && !in_array($t22,$paichu_arr)){
                        $fangfa_score = 1+$fangfa_score;
                        $fangfa_content[] = [
                            'cx_id'=>$t22,
                            'cx_type'=>'错写',
                            'cx_name'=>Fault::where(['fault_id'=>$t22])->value('name'),
                        ];
                    }
                }
                //排除方法 end

                //可能原因 start
                //循环固定的可能原因，匹配作答的可能原因
                foreach ($yy_arr as $k31 =>$t31){
                    if(!in_array($t31,$yuanyin_arr)){
                        // echo $t31."<br/>";
                        $yuanyin_socre = 1+$yuanyin_socre;
                        $yuanyin_content[] = [
                            'cx_id'=>$t31,
                            'cx_type'=>'少写',
                            'cx_name'=>Fault::where(['fault_id'=>$t31])->value('name'),
                        ];
                    }
                }
                //循环作答的排除方法，，匹配固定的排除方法
                foreach ($yuanyin_arr as $k32 =>$t32){
                    if(!empty($t32) && !in_array($t32,$yy_arr)){
                        $yuanyin_socre = 1+$yuanyin_socre;
                        $yuanyin_content[] = [
                            'cx_id'=>$t32,
                            'cx_type'=>'错写',
                            'cx_name'=>Fault::where(['fault_id'=>$t32])->value('name'),
                        ];
                    }
                }
                $uniqueArray = array_unique($yuanyin_arr);
                $diff = array_diff_assoc($yuanyin_arr, $uniqueArray);
                if(!empty($diff)){
                    foreach ($diff as $k323 =>$t323){
                        $yuanyin_socre = 1+$yuanyin_socre;
                        $yuanyin_content[] = [
                            'cx_id'=>$t323,
                            'cx_type'=>'错写',
                            'cx_name'=>Fault::where(['fault_id'=>$t323])->value('name'),
                        ];
                    }
                }
                //可能原因 end
                
                
                $weixiu_score = $xianxian_score+$yuanyin_socre+$buwei_score+$fangfa_score;
                if($weixiu_score>15){
                    $weixiu_score = 15;
                }

                //是否超时
                $overtime_fen = intval((time()-$info['endtime']) / 60);
                if($overtime_fen>=10){
                    $overtime_score = 10;
                }else if($overtime_fen>0 && $overtime_fen<10){
                    $overtime_score = $overtime_fen ;
                }else{
                    $overtime_score =0;
                }
                $fault_score = $score-$fault_one_score-$fault_two_score-$fault_three_score-$weixiu_score-$overtime_score;

                //更新考试结果表
                $params['exam_id'] = $ids;
                $params['total'] = $fault_score;//得分;
                $params['fault_one_score'] = $fault_one_score;//得分;
                $params['fault_two_score'] = $fault_two_score;//得分;
                $params['fault_three_score'] = $fault_three_score;//得分;

                $params['xianxian_score'] = $xianxian_score;//得分;
                $params['xianxian_content'] = json_encode($xianxian_content);//错题;

                $params['yuanyin_socre'] = $yuanyin_socre;//得分;
                $params['yuanyin_content'] = json_encode($yuanyin_content);//错题;

                $params['buwei_score'] = $buwei_score;//得分;
                $params['buwei_content'] = json_encode($buwei_content);//错题;

                $params['fangfa_score'] = $fangfa_score;//得分;
                $params['fangfa_content'] = json_encode($fangfa_content);//错题;
//                
                $params['overtime_score'] = $overtime_score;//得分;
                Db::name('real_exam_score')->insert($params);

                $this->exam_model->where(['exam_id'=>$ids])->update(['total_score'=>$fault_score]);

                Db::commit();
            } catch (ValidateException|PDOException|Exception $e) {
                Db::rollback();
                $this->error($e->getMessage());
            }
            if ($result === false) {
                $this->error(__('No rows were inserted'));
            }

            $this->success('交卷成功','/admin/student/exercise/analysis/ids/'.$row['exam_id']);
        }
        $isloading =1;
        if(empty($row->other_replace)){
            $isloading = 0;
            $row->other_replace = '[{"fault_id":"","request_status":"0"}]';
        }
        $row->other_report_text = !empty($row->other_report) ? json_decode($row->other_report,true) : [];

        $row->limit_duration = $this->model->where(['exam_collection_id'=>$row['exam_collection_id']])->value('limit_duration');
        if(empty($row->starttime)){
            $timer = 60*$row->limit_duration;
        }else{
            $timer = $row->limit_duration*60 - abs(time() - $row->starttime);
        }


        $departmentdata = [];
        $departmentdata = Fault::where(['replace_part'=>1,'sim_type'=>$row->sim_type])->order('fault_id asc')->select();


        $this->view->assign('departmentdata', $departmentdata);
        //故障现象
        $xianxiang = Fault::where(['fault_type'=>1,'sim_type'=>$row->sim_type])->select();
        $this->view->assign('xianxiang', $xianxiang);

        $row->replace_list =Db::name('real_exam_comp_request')->where(['exam_id'=>$ids])->select();
        $this->assignConfig('ids',$ids);
        $this->assignConfig('timer',$timer);
        $this->assignConfig('isloading',$isloading);

        $this->view->assign('row', $row);
        return $this->view->fetch();
    }

    public function analysis($ids = null)
    {
        if ($this->request->isPost()) {
            $params = $this->request->post('row/a');
            //计算总分到学员考试表
            $this->success('操作成功','/admin/student/exercise/index');
        }
        $row = Db::name('real_exam_score')->where('exam_id', $ids)->find();
        if (!$row) {
            $this->error(__('No Results were found'));
        }
        $rows = $this->exam_model->get($ids);
        $row['seat_id'] = $rows->seat_id;
        $row['user_nickname'] = $rows->user_nickname;
        $row['user_username'] = $rows->user_username;
        $row['user_depart_id'] = $rows->user_depart_id;
        $row['user_depart_name'] = Department::where('id',$rows->user_depart_id)->value('name');
        $row['start_time'] = $rows->start_time;
        $row['end_time'] = $rows->end_time;


        $fault_list = Db::name('real_exam_fault')->where(['exam_id'=>$ids,'flag'=>1])->select();
        $row['fault_name_one'] = !empty($fault_list[0]['fault_id']) ? Fault::where('fault_id',$fault_list[0]['fault_id'])->value('name'):'';
        $row['fault_name_two'] = !empty($fault_list[1]['fault_id']) ? Fault::where('fault_id',$fault_list[1]['fault_id'])->value('name'):'';
        $row['fault_name_three'] = !empty($fault_list[2]['fault_id']) ? Fault::where('fault_id',$fault_list[2]['fault_id'])->value('name'):'';

        $row['xianxian_content_name'] = '';
        if(!empty($row['xianxian_content']))
        {
            $row['xianxian_content_name'] = json_decode($row['xianxian_content'],true);
        }
        $row['yuanyin_content_name'] = '';
        if(!empty($row['yuanyin_content']))
        {
            $row['yuanyin_content_name'] = json_decode($row['yuanyin_content'],true);
        }
        $row['buwei_content_name'] = '';
        if(!empty($row['buwei_content']))
        {
            $row['buwei_content_name'] = json_decode($row['buwei_content'],true);
        }
        $row['fangfa_content_name'] = '';
        if(!empty($row['fangfa_content']))
        {
            $row['fangfa_content_name'] = json_decode($row['fangfa_content'],true);
        }

        $other_report = !empty($row['other_report']) ? json_decode($row['other_report'],true):[];
        $this->view->assign('other_report', $other_report);
        $this->view->assign('row', $row);
        return $this->view->fetch();
    }

}
